import os
import shutil
import sys
import tarfile
import tempfile

import genomicsdb

import pandas as pd

def run_test():
        genomicsdb.version()

        gdb = genomicsdb.connect("ws", "callset_t0_1_2.json", "vid.json", "chr1_10MB.fasta.gz", ["DP"], 40)
        try:
                gdb.query_variant_calls()
        except Exception as e:
                print(e)

        gdb = genomicsdb.connect("ws", "callset_t0_1_2.json", "vid.json", "chr1_10MB.fasta.gz", ["DP"], 40)
        list = gdb.query_variant_calls("t0_1_2", [(0,1000000000)], [(0,3)])
        print(list)

        list = gdb.query_variant_calls("t0_1_2", [(0,13000), (13000, 1000000000)], [(0,3)])
        print(list)
        x,y,calls = zip(*list)
        print(pd.DataFrame(calls[0]))

        gdb.query_variant_calls("t0_1_2", [(0,1000000000)])
        gdb.query_variant_calls("t0_1_2")

        output = "out.vcf.gz"
        gdb.to_vcf("t0_1_2", [(0,15000)], output=output, output_format="z")
        if not os.path.exists(output):
            print(output+" NOT FOUND")
        if not os.path.exists(output+".tbi"):
            print("Index file to "+output+" NOT FOUND")

tmp_dir = tempfile.TemporaryDirectory().name

if not os.path.exists(tmp_dir):
        os.makedirs(tmp_dir)
else:
        sys.exit("Aborting as temporary directory seems to exist!")

tar = tarfile.open("test/inputs/sanity.test.tgz")
tar.extractall(tmp_dir)

os.chdir(tmp_dir)

run_test()

shutil.rmtree(tmp_dir)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# pathds documentation build configuration file, created by
# sphinx-quickstart on Fri Jun  9 13:47:02 2017.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another
# directory, add these directories to sys.path here. If the directory is
# relative to the documentation root, use os.path.abspath to make it
# absolute, like shown here.
#
import os
import sys
sys.path.insert(0, os.path.abspath('..'))

import sphinx_bootstrap_theme

import genomicsdb

# -- General configuration ---------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.autodoc', 'sphinx.ext.viewcode', 'sphinx.ext.napoleon', 'm2r']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
source_suffix = ['.rst', '.md']

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'GenomicsDB Python API'
copyright = u"2020, Omics Data Automation"
author = u"Kevin Matlock"

# The version info for the project you're documenting, acts as replacement
# for |version| and |release|, also used in various other places throughout
# the built documents.
#
# The short X.Y version.
version = genomicsdb.version()
# The full version, including alpha/beta/rc tags.
release = genomicsdb.version()

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False


# -- Options for HTML output -------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'bootstrap'
html_theme_path = sphinx_bootstrap_theme.get_html_theme_path()

# Theme options are theme-specific and customize the look and feel of a
# theme further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']


# -- Options for HTMLHelp output ---------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'genomicsdb_pythondoc'


# -- Options for LaTeX output ------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
    'printindex': r'\footnotesize\raggedright\printindex',
    'extraclassoptions': 'openany,oneside',
    'preamble': r'''
        %%%%%%%%%%%%%%%%%%%% Meher %%%%%%%%%%%%%%%%%%
        %%%add number to subsubsection 2=subsection, 3=subsubsection
        %%% below subsubsection is not good idea.
        \setcounter{secnumdepth}{3}
        %
        %%%% Table of content upto 2=subsection, 3=subsubsection
        \setcounter{tocdepth}{2}

        \usepackage{amsmath,amsfonts,amssymb,amsthm}
        \usepackage{graphicx}

        %%% reduce spaces for Table of contents, figures and tables
        %%% it is used "\addtocontents{toc}{\vskip -1.2cm}" etc. in the document
        \usepackage[notlot,nottoc,notlof]{}

        \usepackage{color}
        \usepackage{transparent}
        \usepackage{eso-pic}

        %% spacing between line
        \usepackage{setspace}
        %%%%\onehalfspacing
        %%%%\doublespacing
        \singlespacing


        %%%%%%%%%%% datetime
        \usepackage{datetime}

        \newdateformat{MonthYearFormat}{%
            \monthname[\THEMONTH], \THEYEAR}


        %% RO, LE will not work for 'oneside' layout.
        %% Change oneside to twoside in document class
        \usepackage{fancyhdr}
        \pagestyle{fancy}
        \fancyhf{}

        %%% Alternating Header for oneside
        \fancyhead[L]{\ifthenelse{\isodd{\value{page}}}{ \small \nouppercase{\leftmark} }{}}
        \fancyhead[R]{\ifthenelse{\isodd{\value{page}}}{}{ \small \nouppercase{\rightmark} }}

        %%% Alternating Header for two side
        %\fancyhead[RO]{\small \nouppercase{\rightmark}}
        %\fancyhead[LE]{\small \nouppercase{\leftmark}}

        %% for oneside: change footer at right side. If you want to use Left and right then use same as header defined above.
        \fancyfoot[R]{\ifthenelse{\isodd{\value{page}}}{{\tiny Meher Krishna Patel} }{\href{http://pythondsp.readthedocs.io/en/latest/pythondsp/toc.html}{\tiny PythonDSP}}}

        %%% Alternating Footer for two side
        %\fancyfoot[RO, RE]{\scriptsize Meher Krishna Patel (mekrip@gmail.com)}

        %%% page number
        \fancyfoot[CO, CE]{\thepage}

        \renewcommand{\headrulewidth}{0.5pt}
        \renewcommand{\footrulewidth}{0.5pt}

        \RequirePackage{tocbibind} %%% comment this to remove page number for following
        \addto\captionsenglish{\renewcommand{\contentsname}{Table of contents}}
        \addto\captionsenglish{\renewcommand{\listfigurename}{List of figures}}
        \addto\captionsenglish{\renewcommand{\listtablename}{List of tables}}
        % \addto\captionsenglish{\renewcommand{\chaptername}{Chapter}}


        %%reduce spacing for itemize
        \usepackage{enumitem}
        \setlist{nosep}

        %%%%%%%%%%% Quote Styles at the top of chapter
        \usepackage{epigraph}
        \setlength{\epigraphwidth}{0.8\columnwidth}
        \newcommand{\chapterquote}[2]{\epigraphhead[60]{\epigraph{\textit{#1}}{\textbf {\textit{--#2}}}}}
        %%%%%%%%%%% Quote for all places except Chapter
        \newcommand{\sectionquote}[2]{{\quote{\textit{``#1''}}{\textbf {\textit{--#2}}}}}
    ''',
    'maketitle': r'''
        \pagenumbering{Roman} %%% to avoid page 1 conflict with actual page 1

        \begin{titlepage}
            \centering

            \vspace*{40mm} %%% * is used to give space from top
            \textbf{\Huge {GenomicsDB Python API}}

            \vspace{0mm}
            \begin{figure}[!h]
                \centering
                \includegraphics[scale=0.7]{logo.pdf}
            \end{figure}

            \vspace{0mm}

            \small Created on : February, 2020

            \vspace*{0mm}
            \small  Last updated : \MonthYearFormat\today


            %% \vfill adds at the bottom
            \vfill
            \small \textit{Utilizes the C version of GenomicsDB found on }{\href{https://github.com/GenomicsDB/GenomicsDB}{Github}}
        \end{titlepage}

        \pagenumbering{arabic}

        '''
}

latex_logo = '_static/logo.pdf'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'genomicsdb_python.tex',
     u'GenomicsDB Python API Documentation',
     u'Omics Data Automation', 
     'manual'),
]


# -- Options for manual page output ------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'genomicsdb_python',
     u'GenomicsDB Python API Documentation',
     [author], 1)
]


# -- Options for Texinfo output ----------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'genomicsdb_python',
     u'GenomicsDB Python API Documentation',
     author,
     'genomicsdb_python',
     'Python API for Genomics Data Store.',
     'Miscellaneous'),
]






# distutils: language = c++
# cython: language_level=3

cimport cython

from libcpp.utility cimport pair
from libcpp.string cimport string
from libcpp.vector cimport vector
from libc.stdint cimport (int64_t, uint64_t, uintptr_t)


from cpython.version cimport PY_MAJOR_VERSION

from cpython.bytes cimport (PyBytes_GET_SIZE,
                            PyBytes_AS_STRING,
                            PyBytes_Size,
                            PyBytes_FromString,
                            PyBytes_FromStringAndSize)

cdef unicode to_unicode(s):
    if type(s) is unicode:
        # Fast path for most common case(s).
        return <unicode>s
    elif PY_MAJOR_VERSION < 3 and isinstance(s, bytes):
        # Only accept byte strings as text input in Python 2.x, not in Py3.
        return (<bytes>s).decode('ascii')
    elif isinstance(s, unicode):
        # We know from the fast path above that 's' can only be a subtype here.
        # An evil cast to <unicode> might still work in some(!) cases,
        # depending on what the further processing does.  To be safe,
        # we can always create a copy instead.
        return unicode(s)
    else:
        raise TypeError("Could not convert to unicode.")

cdef string as_string(s):
    return PyBytes_AS_STRING(to_unicode(s).encode('UTF-8'))

cdef vector[string] as_vector(l):
    cdef vector[string] v
    if l is not None:
        for s in l:
            v.push_back(as_string(s))
    return v

cdef genomicsdb_ranges_t as_ranges(l_ranges):
    cdef vector[pair[int64_t, int64_t]] ranges
    if l_ranges is not None and len(l_ranges) > 0:
        for low, high in l_ranges:
            ranges.push_back(pair[int64_t, int64_t](low, high))
    return ranges



import unittest

import os
import shutil
import sys
import tarfile
import tempfile

import genomicsdb

class TestGenomicsDBVersion(unittest.TestCase):
    def test_version(self):
        version = genomicsdb.version()
        self.assertIsInstance(version, str)
        self.assertTrue(len(version) > 0)
        # Should contain major.minor.patch in version string
        version_components = version.split('.')
        self.assertTrue(len(version_components) == 3)
        self.assertTrue(int(version_components[0]) > 0)
        self.assertTrue(int(version_components[1]) >= 0)

if __name__ == '__main__':
    unittest.main()



# distutils: language = c++
# cython: language_level=3

include "utils.pxi"

import tempfile

def version():
    """Print out version of the GenomicsDB native library
    
    Returns
    -------
    str
        GenomicsDB Version
    """
    version_string = genomicsdb_version().decode("ascii")
    return version_string

class GenomicsDBException(Exception):
    pass

def connect(workspace,
            callset_mapping_file = "callset.json",
            vid_mapping_file = "vidmap.json",
            reference_genome = None,
            attributes = None,
            segment_size = None):
    """Connect to an existing GenomicsDB Workspace.
    
    Parameters
    ----------
    workspace : str
        Path to the GenomicsDB workspace.
    callset_mapping_file : str, optional
        Path to a json file describing callset mappings, by default "callset.json"
    vid_mapping_file : str, optional
        Path to a json file describing vid mappings, by default "vidmap.json"
    reference_genome : str, optional
        Path to the reference genome file, by default None
    attributes : list, optional
        List of attributes to be queried, by default None. All attributes will be queried if None.
    segment_size : int, optional
        Segment Size, by default None. Allow GenomicsDB to configure one.
    
    Returns
    -------
    GenomicsDB
        GenomicsDB instance ready for queries
    
    Raises
    ------
    GenomicsDBException
         On failure to connect to the native GenomicsDB library
    """    
    try:
        return _GenomicsDB(workspace, callset_mapping_file, vid_mapping_file,
                           reference_genome, attributes, segment_size)
    except:
        raise GenomicsDBException("Failed to connect to the native GenomicsDB library")

cdef class _GenomicsDB:
    cdef GenomicsDB* _genomicsdb

    def __cinit__(self):
        self._genomicsdb = NULL

    def __init__(self,
                 workspace,
                 callset_mapping_file,
                 vid_mapping_file,
                 reference_genome = None,
                 attributes = None,
                 segment_size = None):
        cdef string ws = as_string(workspace)
        cdef vector[string] vec = as_vector(attributes)
        if reference_genome is None:
            self._genomicsdb  = new GenomicsDB(as_string(workspace),
                                               as_string(callset_mapping_file),
                                               as_string(vid_mapping_file),
                                               as_string(""))
        if attributes is None:
            self._genomicsdb  = new GenomicsDB(as_string(workspace),
                                               as_string(callset_mapping_file),
                                               as_string(vid_mapping_file),
                                               as_string(reference_genome))
        elif segment_size is None:
            self._genomicsdb  = new GenomicsDB(as_string(workspace),
                                               as_string(callset_mapping_file),
                                               as_string(vid_mapping_file),
                                               as_string(reference_genome),
                                               as_vector(attributes))
        else:
            self._genomicsdb  = new GenomicsDB(as_string(workspace),
                                               as_string(callset_mapping_file),
                                               as_string(vid_mapping_file),
                                               as_string(reference_genome),
                                               as_vector(attributes),
                                               segment_size)

    def query_variant_calls(self,
                            array=None,
                            column_ranges=None,
                            row_ranges=None):
        """ Query for variant calls from the GenomicsDB workspace using array, column_ranges and row_ranges for subsetting """

        cdef list variant_calls = []
        cdef VariantCallProcessor processor
        processor.set_root(variant_calls)
        if array is None:
            self._genomicsdb.query_variant_calls()
        elif column_ranges is None:
            self._genomicsdb.query_variant_calls(processor, as_string(array))
        elif row_ranges is None:
            self._genomicsdb.query_variant_calls(processor, as_string(array),
                                                 as_ranges(column_ranges))
        else:
            self._genomicsdb.query_variant_calls(processor, as_string(array),
                                                 as_ranges(column_ranges),
                                                 as_ranges(row_ranges))
        return variant_calls

    def to_vcf(self,
               array=None,
               column_ranges=None,
               row_ranges=None,
               output=None,
               output_format=None,
               overwrite=False):
        """ Generate vcf from the GenomicsDB workspace using array, column_ranges and row_ranges for subsetting """

        if output is None:
            output = ""
        if output_format is None:
            output_format = ""
        if array is None:
            self._genomicsdb.generate_vcf(as_string(output),
                                          as_string(output_format),
                                          overwrite)
        else:
            self._genomicsdb.generate_vcf(as_string(array),
                                          as_ranges(column_ranges),
                                          as_ranges(row_ranges),
                                          as_string(output),
                                          as_string(output_format),
                                          overwrite)


    def __dealloc__(self):
        if self._genomicsdb != NULL:
            del self._genomicsdb




# distutils: language = c++
# cython: language_level=3

from libcpp.pair cimport pair
from libcpp.string cimport string
from libcpp.vector cimport vector
from libcpp.functional cimport function
from libc.stdint cimport (int32_t, uint32_t, int64_t, uint64_t, uintptr_t, INT64_MAX)
from cpython cimport (PyObject, PyList_New)

cdef extern from "genomicsdb.h":
    cdef string genomicsdb_version()

#   GenomicsDB typedefs
    ctypedef pair[uint64_t, uint64_t] interval_t;

    cdef struct genomic_interval_t:
        string contig_name
        interval_t interval
        genomic_interval_t(string, interval_t)

    ctypedef pair[string, string] genomic_field_t

    cdef struct genomicsdb_variant_t:
        pass

    cdef struct genomicsdb_variant_call_t:
        pass

    ctypedef vector[pair[int64_t, int64_t]] genomicsdb_ranges_t;

    cdef cppclass GenomicsDBResults[T]:
        GenomicsDBResults(vector[T]*) except +
        size_t size()
        const T* at(size_t)
        inline const T* next()
        void free()

    ctypedef GenomicsDBResults[genomicsdb_variant_t] GenomicsDBVariants
    ctypedef GenomicsDBResults[genomicsdb_variant_call_t] GenomicsDBVariantCalls

    cdef cppclass GenomicsDBVariantCallProcessor:
        GenomicsDBVariantCallProcessor() except +
        void process(interval_t)
        void process(uint32_t, genomic_interval_t, vector[genomic_field_t])

#   GenomicsDB Class

    cdef cppclass GenomicsDB:
        GenomicsDB(string, string, string, string, vector[string], uint64_t) except +
        GenomicsDB(string, string, string, string, vector[string]) except +
        GenomicsDB(string, string, string, string) except +
        GenomicsDB(string, string, int) except +
        GenomicsDB(string, string) except +
        GenomicsDB(string) except +
        GenomicsDBVariants query_variants(string, genomicsdb_ranges_t, genomicsdb_ranges_t) except +
        GenomicsDBVariants query_variants()
        GenomicsDBVariantCalls query_variant_calls(GenomicsDBVariantCallProcessor, string, genomicsdb_ranges_t, genomicsdb_ranges_t) except +
        GenomicsDBVariantCalls query_variant_calls(string, genomicsdb_ranges_t, genomicsdb_ranges_t) except +
        GenomicsDBVariantCalls query_variant_calls(GenomicsDBVariantCallProcessor, string, genomicsdb_ranges_t) except +
        GenomicsDBVariantCalls query_variant_calls(string, genomicsdb_ranges_t) except +
        GenomicsDBVariantCalls query_variant_calls(GenomicsDBVariantCallProcessor, string) except +
        GenomicsDBVariantCalls query_variant_calls(string) except +
        GenomicsDBVariantCalls query_variant_calls(GenomicsDBVariantCallProcessor) except +
        GenomicsDBVariantCalls query_variant_calls() except +
        void generate_vcf(string, genomicsdb_ranges_t, genomicsdb_ranges_t, string, string, bool) except +
        void generate_vcf(string, genomicsdb_ranges_t, genomicsdb_ranges_t, string, string) except +
        void generate_vcf(string, genomicsdb_ranges_t, genomicsdb_ranges_t, string) except +
        void generate_vcf(string, genomicsdb_ranges_t, genomicsdb_ranges_t) except +
        void generate_vcf(string, string, bool) except +
        void generate_vcf(string, string) except +
        void generate_vcf(string) except +
        void generate_vcf() except +
        pass

#   GenomicsDB Helper Utilities

    cdef interval_t get_interval(genomicsdb_variant_t*)
    cdef interval_t get_interval(genomicsdb_variant_call_t*)
    cdef genomic_interval_t get_genomic_interval(genomicsdb_variant_t*)
    cdef genomic_interval_t get_genomic_interval(genomicsdb_variant_call_t*)
    cdef vector[genomic_field_t] get_genomic_fields(string, genomicsdb_variant_t*)
    cdef vector[genomic_field_t] get_genomic_fields(string, genomicsdb_variant_call_t*)
    cdef GenomicsDBVariantCalls get_variant_calls(genomicsdb_variant_t*)
    cdef int64_t get_row(genomicsdb_variant_call_t*)
    pass

cdef extern from "genomicsdb_processor.h":
    cdef cppclass VariantCallProcessor(GenomicsDBVariantCallProcessor):
        VariantCallProcessor() except +
        void set_root(object)
        void process(interval_t) except +
        void process(uint32_t, genomic_interval_t, vector[genomic_field_t]) except +
        void finalize() except +
        pass




from __future__ import absolute_import, print_function 

from setuptools import setup, Extension, find_packages
from pkg_resources import resource_filename

import os
import shutil
import glob
import sys
import logging

# Directory where a copy of the CPP compiled object is found.
GENOMICSDB_LOCAL_DATA_DIR = 'genomicsdb_data'

# Specify genomicsdb install location via "--with-genomicsdb=<genomicsdb_install_path>" command line arg
GENOMICSDB_INSTALL_PATH = os.getenv('GENOMICSDB_HOME', default = '/usr/local')
args = sys.argv[:]
for arg in args:
	if arg.find('--with-genomicsdb=') == 0:
		GENOMICSDB_INSTALL_PATH = os.path.expanduser(arg.split('=')[1])
		sys.argv.remove(arg)

print('Compiled GenomicsDB Install Path: {}'.format(GENOMICSDB_INSTALL_PATH))

GENOMICSDB_INCLUDE_DIR = os.path.join(GENOMICSDB_INSTALL_PATH, "include")
GENOMICSDB_LIB_DIR = os.path.join(GENOMICSDB_INSTALL_PATH, "lib")

rpath = []
for arg in args:
	if arg.find('--with-libs') == 0:
		glob_paths = [os.path.join(GENOMICSDB_LIB_DIR, e) for e in ['lib*genomicsdb*.so','lib*genomicsdb*.dylib']]
		lib_paths = []
		for paths in glob_paths:
			lib_paths.extend(glob.glob(paths))
		print('Adding the following libraries to the GenomicsDB Package :')
		print(*lib_paths, sep="\n")

		dst = os.path.join(GENOMICSDB_LOCAL_DATA_DIR, 'lib')
		if os.path.isdir(dst):
			shutil.rmtree(dst)
		os.makedirs(dst)
		for lib_path in lib_paths:
			print('Copying {0} to {1}'.format(lib_path, dst))
			shutil.copy(lib_path, dst)
		rpath = ['$ORIGIN/' + dst]

		sys.argv.remove(arg)

genomicsdb_extension=Extension(
	"genomicsdb",
	language="c++",
	sources=["src/genomicsdb.pyx", "src/genomicsdb_processor.cpp"],
	libraries=["tiledbgenomicsdb"],
	include_dirs=[GENOMICSDB_INCLUDE_DIR],
	library_dirs=[GENOMICSDB_LIB_DIR],
	runtime_library_dirs= rpath,
	extra_compile_args=["-std=c++11"]
)

with open('README.md') as f:
	long_description = f.read()

setup(
	name='genomicsdb',
	description='Experimental Python Bindings to GenomicsDB',
	long_description=long_description,
	long_description_content_type='text/markdown',
	author='GenomicsDB.org',
	author_email='support@genomicsdb.org',
	maintainer='GenomicsDB.org',
	maintainer_email='support@genomicsdb.org',
	license='MIT',
	ext_modules=[genomicsdb_extension],
	setup_requires=['cython>=0.27'],
	install_requires=[
		'numpy>=1.7',
		'wheel>=0.30'],
	packages = find_packages(),
	keywords=['genomics', 'genomicsdb', 'variant'],
	include_package_data=True,
	version = '0.0.6.1',
	classifiers=[
		'Development Status :: 3 - Alpha',
		'Intended Audience :: Developers',
		'Intended Audience :: Information Technology',
		'Intended Audience :: Science/Research',
		'License :: OSI Approved :: MIT License',
		'Programming Language :: Python',
		'Topic :: Software Development :: Libraries :: Python Modules',
		'Operating System :: POSIX :: Linux',
		'Operating System :: MacOS :: MacOS X',
		'Programming Language :: Python :: 3',
		'Programming Language :: Python :: 3.6',
	],
)

